import { Controller } from '@nestjs/common';
import { QueryBus } from '@nestjs/cqrs';
import { GrpcMethod, RpcException } from '@nestjs/microservices';
import { DecisionRequest } from '../../domain/dtos/decision.request';
import { DecisionQuery } from '../../queries/decision.query';
import { DecisionResult } from '../secondaries/decision-result';

@Controller()
export class AuthorizationController {
  constructor(private readonly _queryBus: QueryBus) {}

  @GrpcMethod('AuthorizationService', 'Decide')
  async decide(data: DecisionRequest): Promise<DecisionResult> {
    try {
      const decision: boolean = await this._queryBus.execute(
        new DecisionQuery(data.uuid, data.domain, data.action, data.context),
      );
      return {
        allow: decision,
      };
    } catch (e) {
      throw new RpcException({
        code: 7,
        message: 'Permission denied',
      });
    }
  }
}
